mirror of
https://github.com/NVIDIA/cuda-samples.git
synced 2026-10-13 00:08:26 +08:00
CUDA 13.2 samples update (#432)
- Added Python samples for CUDA Python 1.0 release - Renamed top-level `Samples` directory to `cpp` to accommodate Python samples.
This commit is contained in:
18
cpp/3_CUDA_Features/cudaCompressibleMemory/.vscode/c_cpp_properties.json
vendored
Normal file
18
cpp/3_CUDA_Features/cudaCompressibleMemory/.vscode/c_cpp_properties.json
vendored
Normal file
@@ -0,0 +1,18 @@
|
||||
{
|
||||
"configurations": [
|
||||
{
|
||||
"name": "Linux",
|
||||
"includePath": [
|
||||
"${workspaceFolder}/**",
|
||||
"${workspaceFolder}/../../../Common"
|
||||
],
|
||||
"defines": [],
|
||||
"compilerPath": "/usr/local/cuda/bin/nvcc",
|
||||
"cStandard": "gnu17",
|
||||
"cppStandard": "gnu++14",
|
||||
"intelliSenseMode": "linux-gcc-x64",
|
||||
"configurationProvider": "ms-vscode.makefile-tools"
|
||||
}
|
||||
],
|
||||
"version": 4
|
||||
}
|
||||
7
cpp/3_CUDA_Features/cudaCompressibleMemory/.vscode/extensions.json
vendored
Normal file
7
cpp/3_CUDA_Features/cudaCompressibleMemory/.vscode/extensions.json
vendored
Normal file
@@ -0,0 +1,7 @@
|
||||
{
|
||||
"recommendations": [
|
||||
"nvidia.nsight-vscode-edition",
|
||||
"ms-vscode.cpptools",
|
||||
"ms-vscode.makefile-tools"
|
||||
]
|
||||
}
|
||||
42
cpp/3_CUDA_Features/cudaCompressibleMemory/CMakeLists.txt
Normal file
42
cpp/3_CUDA_Features/cudaCompressibleMemory/CMakeLists.txt
Normal file
@@ -0,0 +1,42 @@
|
||||
cmake_minimum_required(VERSION 3.20)
|
||||
|
||||
list(APPEND CMAKE_MODULE_PATH "${CMAKE_CURRENT_SOURCE_DIR}/../../../cmake/Modules")
|
||||
|
||||
project(cudaCompressibleMemory LANGUAGES C CXX CUDA)
|
||||
|
||||
find_package(CUDAToolkit REQUIRED)
|
||||
|
||||
set(CMAKE_POSITION_INDEPENDENT_CODE ON)
|
||||
|
||||
set(CMAKE_CUDA_ARCHITECTURES 75 80 86 87 89 90 100 110 120)
|
||||
set(CMAKE_CUDA_FLAGS "${CMAKE_CUDA_FLAGS} -Wno-deprecated-gpu-targets")
|
||||
if(ENABLE_CUDA_DEBUG)
|
||||
set(CMAKE_CUDA_FLAGS "${CMAKE_CUDA_FLAGS} -G") # enable cuda-gdb (may significantly affect performance on some targets)
|
||||
else()
|
||||
set(CMAKE_CUDA_FLAGS "${CMAKE_CUDA_FLAGS} -lineinfo") # add line information to all builds for debug tools (exclusive to -G option)
|
||||
endif()
|
||||
|
||||
# Include directories and libraries
|
||||
include_directories(../../../Common)
|
||||
|
||||
# Source file
|
||||
# Add target for cudaCompressibleMemory
|
||||
add_executable(cudaCompressibleMemory compMalloc.cpp saxpy.cu)
|
||||
|
||||
target_compile_options(cudaCompressibleMemory PRIVATE $<$<COMPILE_LANGUAGE:CUDA>:--extended-lambda>)
|
||||
|
||||
target_compile_features(cudaCompressibleMemory PRIVATE cxx_std_17 cuda_std_17)
|
||||
|
||||
set_target_properties(cudaCompressibleMemory PROPERTIES CUDA_SEPARABLE_COMPILATION ON)
|
||||
|
||||
target_include_directories(cudaCompressibleMemory PRIVATE
|
||||
${CUDAToolkit_INCLUDE_DIRS}
|
||||
)
|
||||
|
||||
target_link_libraries(cudaCompressibleMemory PRIVATE
|
||||
CUDA::cuda_driver
|
||||
)
|
||||
|
||||
# Include installation configuration
|
||||
include(${CMAKE_CURRENT_SOURCE_DIR}/../../../cmake/InstallSamples.cmake)
|
||||
setup_samples_install()
|
||||
35
cpp/3_CUDA_Features/cudaCompressibleMemory/README.md
Normal file
35
cpp/3_CUDA_Features/cudaCompressibleMemory/README.md
Normal file
@@ -0,0 +1,35 @@
|
||||
# cudaCompressibleMemory - CUDA Compressible Memory
|
||||
|
||||
## Description
|
||||
|
||||
This sample demonstrates the compressible memory allocation using cuMemMap API.
|
||||
|
||||
## Key Concepts
|
||||
|
||||
CUDA Driver API, Compressible Memory, MMAP
|
||||
|
||||
## Supported SM Architectures
|
||||
|
||||
[SM 5.0 ](https://developer.nvidia.com/cuda-gpus) [SM 5.2 ](https://developer.nvidia.com/cuda-gpus) [SM 5.3 ](https://developer.nvidia.com/cuda-gpus) [SM 6.0 ](https://developer.nvidia.com/cuda-gpus) [SM 6.1 ](https://developer.nvidia.com/cuda-gpus) [SM 7.0 ](https://developer.nvidia.com/cuda-gpus) [SM 7.2 ](https://developer.nvidia.com/cuda-gpus) [SM 7.5 ](https://developer.nvidia.com/cuda-gpus) [SM 8.0 ](https://developer.nvidia.com/cuda-gpus) [SM 8.6 ](https://developer.nvidia.com/cuda-gpus) [SM 8.7 ](https://developer.nvidia.com/cuda-gpus) [SM 8.9 ](https://developer.nvidia.com/cuda-gpus) [SM 9.0 ](https://developer.nvidia.com/cuda-gpus)
|
||||
|
||||
## Supported OSes
|
||||
|
||||
Linux, Windows, QNX
|
||||
|
||||
## Supported CPU Architecture
|
||||
|
||||
x86_64, aarch64
|
||||
|
||||
## CUDA APIs involved
|
||||
|
||||
### [CUDA Driver API](http://docs.nvidia.com/cuda/cuda-driver-api/index.html)
|
||||
cuMemGetAllocationPropertiesFromHandle, cuMemCreate, cuDeviceGetAttribute, cuCtxGetDevice, cuMemGetAllocationGranularity, cuMemAddressFree, cuMemUnmap, cuMemMap, cuMemRelease, cuMemAddressReserve, cuMemSetAccess
|
||||
|
||||
### [CUDA Runtime API](http://docs.nvidia.com/cuda/cuda-runtime-api/index.html)
|
||||
cudaMemcpy, cudaEventSynchronize, cudaEventRecord, cudaEventElapsedTime, cudaOccupancyMaxPotentialBlockSize, cudaEventCreate
|
||||
|
||||
## Prerequisites
|
||||
|
||||
Download and install the [CUDA Toolkit](https://developer.nvidia.com/cuda-downloads) for your corresponding platform.
|
||||
|
||||
## References (for more details)
|
||||
115
cpp/3_CUDA_Features/cudaCompressibleMemory/compMalloc.cpp
Normal file
115
cpp/3_CUDA_Features/cudaCompressibleMemory/compMalloc.cpp
Normal file
@@ -0,0 +1,115 @@
|
||||
/* Copyright (c) 2022, NVIDIA CORPORATION. All rights reserved.
|
||||
*
|
||||
* Redistribution and use in source and binary forms, with or without
|
||||
* modification, are permitted provided that the following conditions
|
||||
* are met:
|
||||
* * Redistributions of source code must retain the above copyright
|
||||
* notice, this list of conditions and the following disclaimer.
|
||||
* * Redistributions in binary form must reproduce the above copyright
|
||||
* notice, this list of conditions and the following disclaimer in the
|
||||
* documentation and/or other materials provided with the distribution.
|
||||
* * Neither the name of NVIDIA CORPORATION nor the names of its
|
||||
* contributors may be used to endorse or promote products derived
|
||||
* from this software without specific prior written permission.
|
||||
*
|
||||
* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS ``AS IS'' AND ANY
|
||||
* EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
|
||||
* IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
|
||||
* PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
|
||||
* CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
|
||||
* EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
|
||||
* PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
|
||||
* PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
|
||||
* OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
|
||||
* (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
|
||||
* OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
|
||||
*/
|
||||
|
||||
#include <cuda.h>
|
||||
#include <cuda_runtime_api.h>
|
||||
#include <helper_cuda.h>
|
||||
#include <stdio.h>
|
||||
#include <string.h>
|
||||
|
||||
cudaError_t setProp(CUmemAllocationProp *prop, bool UseCompressibleMemory)
|
||||
{
|
||||
CUdevice currentDevice;
|
||||
if (cuCtxGetDevice(¤tDevice) != CUDA_SUCCESS)
|
||||
return cudaErrorMemoryAllocation;
|
||||
|
||||
memset(prop, 0, sizeof(CUmemAllocationProp));
|
||||
prop->type = CU_MEM_ALLOCATION_TYPE_PINNED;
|
||||
prop->location.type = CU_MEM_LOCATION_TYPE_DEVICE;
|
||||
prop->location.id = currentDevice;
|
||||
|
||||
if (UseCompressibleMemory)
|
||||
prop->allocFlags.compressionType = CU_MEM_ALLOCATION_COMP_GENERIC;
|
||||
|
||||
return cudaSuccess;
|
||||
}
|
||||
|
||||
cudaError_t allocateCompressible(void **adr, size_t size, bool UseCompressibleMemory)
|
||||
{
|
||||
CUmemAllocationProp prop = {};
|
||||
cudaError_t err = setProp(&prop, UseCompressibleMemory);
|
||||
if (err != cudaSuccess)
|
||||
return err;
|
||||
|
||||
size_t granularity = 0;
|
||||
if (cuMemGetAllocationGranularity(&granularity, &prop, CU_MEM_ALLOC_GRANULARITY_MINIMUM) != CUDA_SUCCESS)
|
||||
return cudaErrorMemoryAllocation;
|
||||
size = ((size - 1) / granularity + 1) * granularity;
|
||||
CUdeviceptr dptr;
|
||||
if (cuMemAddressReserve(&dptr, size, 0, 0, 0) != CUDA_SUCCESS)
|
||||
return cudaErrorMemoryAllocation;
|
||||
|
||||
CUmemGenericAllocationHandle allocationHandle;
|
||||
if (cuMemCreate(&allocationHandle, size, &prop, 0) != CUDA_SUCCESS)
|
||||
return cudaErrorMemoryAllocation;
|
||||
|
||||
// Check if cuMemCreate was able to allocate compressible memory.
|
||||
if (UseCompressibleMemory) {
|
||||
CUmemAllocationProp allocationProp = {};
|
||||
cuMemGetAllocationPropertiesFromHandle(&allocationProp, allocationHandle);
|
||||
if (allocationProp.allocFlags.compressionType != CU_MEM_ALLOCATION_COMP_GENERIC) {
|
||||
printf("Could not allocate compressible memory... so waiving execution\n");
|
||||
exit(EXIT_WAIVED);
|
||||
}
|
||||
}
|
||||
|
||||
if (cuMemMap(dptr, size, 0, allocationHandle, 0) != CUDA_SUCCESS)
|
||||
return cudaErrorMemoryAllocation;
|
||||
|
||||
if (cuMemRelease(allocationHandle) != CUDA_SUCCESS)
|
||||
return cudaErrorMemoryAllocation;
|
||||
|
||||
CUmemAccessDesc accessDescriptor;
|
||||
accessDescriptor.location.id = prop.location.id;
|
||||
accessDescriptor.location.type = prop.location.type;
|
||||
accessDescriptor.flags = CU_MEM_ACCESS_FLAGS_PROT_READWRITE;
|
||||
|
||||
if (cuMemSetAccess(dptr, size, &accessDescriptor, 1) != CUDA_SUCCESS)
|
||||
return cudaErrorMemoryAllocation;
|
||||
|
||||
*adr = (void *)dptr;
|
||||
return cudaSuccess;
|
||||
}
|
||||
|
||||
cudaError_t freeCompressible(void *ptr, size_t size, bool UseCompressibleMemory)
|
||||
{
|
||||
CUmemAllocationProp prop = {};
|
||||
cudaError_t err = setProp(&prop, UseCompressibleMemory);
|
||||
if (err != cudaSuccess)
|
||||
return err;
|
||||
|
||||
size_t granularity = 0;
|
||||
if (cuMemGetAllocationGranularity(&granularity, &prop, CU_MEM_ALLOC_GRANULARITY_MINIMUM) != CUDA_SUCCESS)
|
||||
return cudaErrorMemoryAllocation;
|
||||
size = ((size - 1) / granularity + 1) * granularity;
|
||||
|
||||
if (ptr == NULL)
|
||||
return cudaSuccess;
|
||||
if (cuMemUnmap((CUdeviceptr)ptr, size) != CUDA_SUCCESS || cuMemAddressFree((CUdeviceptr)ptr, size) != CUDA_SUCCESS)
|
||||
return cudaErrorInvalidValue;
|
||||
return cudaSuccess;
|
||||
}
|
||||
34
cpp/3_CUDA_Features/cudaCompressibleMemory/compMalloc.h
Normal file
34
cpp/3_CUDA_Features/cudaCompressibleMemory/compMalloc.h
Normal file
@@ -0,0 +1,34 @@
|
||||
/* Copyright (c) 2022, NVIDIA CORPORATION. All rights reserved.
|
||||
*
|
||||
* Redistribution and use in source and binary forms, with or without
|
||||
* modification, are permitted provided that the following conditions
|
||||
* are met:
|
||||
* * Redistributions of source code must retain the above copyright
|
||||
* notice, this list of conditions and the following disclaimer.
|
||||
* * Redistributions in binary form must reproduce the above copyright
|
||||
* notice, this list of conditions and the following disclaimer in the
|
||||
* documentation and/or other materials provided with the distribution.
|
||||
* * Neither the name of NVIDIA CORPORATION nor the names of its
|
||||
* contributors may be used to endorse or promote products derived
|
||||
* from this software without specific prior written permission.
|
||||
*
|
||||
* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS ``AS IS'' AND ANY
|
||||
* EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
|
||||
* IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
|
||||
* PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
|
||||
* CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
|
||||
* EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
|
||||
* PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
|
||||
* PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
|
||||
* OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
|
||||
* (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
|
||||
* OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
|
||||
*/
|
||||
|
||||
#ifndef COMP_MALLOC_H
|
||||
#define COMP_MALLOC_H
|
||||
|
||||
cudaError_t allocateCompressible(void **adr, size_t size, bool UseCompressibleMemory);
|
||||
cudaError_t freeCompressible(void *ptr, size_t size, bool UseCompressibleMemory);
|
||||
|
||||
#endif
|
||||
192
cpp/3_CUDA_Features/cudaCompressibleMemory/saxpy.cu
Normal file
192
cpp/3_CUDA_Features/cudaCompressibleMemory/saxpy.cu
Normal file
@@ -0,0 +1,192 @@
|
||||
/* Copyright (c) 2022, NVIDIA CORPORATION. All rights reserved.
|
||||
*
|
||||
* Redistribution and use in source and binary forms, with or without
|
||||
* modification, are permitted provided that the following conditions
|
||||
* are met:
|
||||
* * Redistributions of source code must retain the above copyright
|
||||
* notice, this list of conditions and the following disclaimer.
|
||||
* * Redistributions in binary form must reproduce the above copyright
|
||||
* notice, this list of conditions and the following disclaimer in the
|
||||
* documentation and/or other materials provided with the distribution.
|
||||
* * Neither the name of NVIDIA CORPORATION nor the names of its
|
||||
* contributors may be used to endorse or promote products derived
|
||||
* from this software without specific prior written permission.
|
||||
*
|
||||
* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS ``AS IS'' AND ANY
|
||||
* EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
|
||||
* IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
|
||||
* PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
|
||||
* CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
|
||||
* EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
|
||||
* PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
|
||||
* PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
|
||||
* OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
|
||||
* (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
|
||||
* OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
|
||||
*/
|
||||
|
||||
//
|
||||
// This sample uses the compressible memory allocation if device supports it
|
||||
// and performs saxpy on it.
|
||||
// Compressible memory may give better performance if the data is amenable to
|
||||
// compression.
|
||||
|
||||
#include <cuda.h>
|
||||
#include <stdio.h>
|
||||
#define CUDA_DRIVER_API
|
||||
#include "compMalloc.h"
|
||||
#include "helper_cuda.h"
|
||||
|
||||
__global__ void saxpy(const float a, const float4 *x, const float4 *y, float4 *z, const size_t n)
|
||||
{
|
||||
for (size_t i = blockIdx.x * blockDim.x + threadIdx.x; i < n; i += gridDim.x * blockDim.x) {
|
||||
const float4 x4 = x[i];
|
||||
const float4 y4 = y[i];
|
||||
z[i] = make_float4(a * x4.x + y4.x, a * x4.y + y4.y, a * x4.z + y4.z, a * x4.w + y4.w);
|
||||
}
|
||||
}
|
||||
|
||||
__global__ void init(float4 *x, float4 *y, const float val, const size_t n)
|
||||
{
|
||||
const float4 val4 = make_float4(val, val, val, val);
|
||||
for (size_t i = blockIdx.x * blockDim.x + threadIdx.x; i < n; i += gridDim.x * blockDim.x) {
|
||||
x[i] = y[i] = val4;
|
||||
}
|
||||
}
|
||||
|
||||
void launchSaxpy(const float a,
|
||||
float4 *x,
|
||||
float4 *y,
|
||||
float4 *z,
|
||||
const size_t n,
|
||||
const float init_val,
|
||||
const bool compressibleZbuf)
|
||||
{
|
||||
cudaEvent_t start, stop;
|
||||
float ms;
|
||||
int blockSize;
|
||||
int minGridSize;
|
||||
dim3 threads, blocks;
|
||||
|
||||
if (!compressibleZbuf) {
|
||||
// We are on config where compressible buffer can only be initialized through cudaMemcpy
|
||||
// hence, x & y buffers are allocated as compressible and initialized via cudaMemcpy
|
||||
// whereas z buffer is allocated as non-compressible.
|
||||
float4 *h_x = (float4 *)malloc(sizeof(float4) * n);
|
||||
float4 *h_y = (float4 *)malloc(sizeof(float4) * n);
|
||||
for (int i = 0; i < n; i++) {
|
||||
h_x[i].x = h_x[i].y = h_x[i].z = h_x[i].w = init_val;
|
||||
h_y[i].x = h_y[i].y = h_y[i].z = h_y[i].w = init_val;
|
||||
}
|
||||
checkCudaErrors(cudaMemcpy(x, h_x, sizeof(float4) * n, cudaMemcpyHostToDevice));
|
||||
checkCudaErrors(cudaMemcpy(y, h_y, sizeof(float4) * n, cudaMemcpyHostToDevice));
|
||||
free(h_x);
|
||||
free(h_y);
|
||||
}
|
||||
else {
|
||||
checkCudaErrors(cudaOccupancyMaxPotentialBlockSize(&minGridSize, &blockSize, (void *)init));
|
||||
threads = dim3(blockSize, 1, 1);
|
||||
blocks = dim3(minGridSize, 1, 1);
|
||||
init<<<blocks, threads>>>(x, y, init_val, n);
|
||||
}
|
||||
|
||||
checkCudaErrors(cudaOccupancyMaxPotentialBlockSize(&minGridSize, &blockSize, (void *)saxpy));
|
||||
threads = dim3(blockSize, 1, 1);
|
||||
blocks = dim3(minGridSize, 1, 1);
|
||||
|
||||
checkCudaErrors(cudaEventCreate(&start));
|
||||
checkCudaErrors(cudaEventCreate(&stop));
|
||||
checkCudaErrors(cudaEventRecord(start));
|
||||
saxpy<<<blocks, threads>>>(a, x, y, z, n);
|
||||
checkCudaErrors(cudaEventRecord(stop));
|
||||
checkCudaErrors(cudaEventSynchronize(stop));
|
||||
checkCudaErrors(cudaEventElapsedTime(&ms, start, stop));
|
||||
|
||||
const size_t size = n * sizeof(float4);
|
||||
printf("Running saxpy with %d blocks x %d threads = %.3f ms %.3f TB/s\n",
|
||||
blocks.x,
|
||||
threads.x,
|
||||
ms,
|
||||
(size * 3) / ms / 1e9);
|
||||
}
|
||||
|
||||
int main(int argc, char **argv)
|
||||
{
|
||||
const size_t n = 10485760;
|
||||
|
||||
if (checkCmdLineFlag(argc, (const char **)argv, "help") || checkCmdLineFlag(argc, (const char **)argv, "?")) {
|
||||
printf("Usage -device=n (n >= 0 for deviceID)\n");
|
||||
exit(EXIT_SUCCESS);
|
||||
}
|
||||
|
||||
findCudaDevice(argc, (const char **)argv);
|
||||
CUdevice currentDevice;
|
||||
checkCudaErrors(cuCtxGetDevice(¤tDevice));
|
||||
|
||||
// Check that the selected device supports virtual memory management
|
||||
int vmm_supported = -1;
|
||||
checkCudaErrors(
|
||||
cuDeviceGetAttribute(&vmm_supported, CU_DEVICE_ATTRIBUTE_VIRTUAL_ADDRESS_MANAGEMENT_SUPPORTED, currentDevice));
|
||||
if (vmm_supported == 0) {
|
||||
printf("Device %d doesn't support Virtual Memory Management, waiving the execution.\n", currentDevice);
|
||||
exit(EXIT_WAIVED);
|
||||
}
|
||||
|
||||
int isCompressionAvailable;
|
||||
checkCudaErrors(cuDeviceGetAttribute(
|
||||
&isCompressionAvailable, CU_DEVICE_ATTRIBUTE_GENERIC_COMPRESSION_SUPPORTED, currentDevice));
|
||||
if (isCompressionAvailable == 0) {
|
||||
printf("Device %d doesn't support Generic memory compression, waiving the execution.\n", currentDevice);
|
||||
exit(EXIT_WAIVED);
|
||||
}
|
||||
|
||||
printf("Generic memory compression support is available\n");
|
||||
|
||||
int major, minor;
|
||||
checkCudaErrors(cuDeviceGetAttribute(&major, CU_DEVICE_ATTRIBUTE_COMPUTE_CAPABILITY_MAJOR, currentDevice));
|
||||
checkCudaErrors(cuDeviceGetAttribute(&minor, CU_DEVICE_ATTRIBUTE_COMPUTE_CAPABILITY_MINOR, currentDevice));
|
||||
float4 *x, *y, *z;
|
||||
const size_t size = n * sizeof(float4);
|
||||
|
||||
// Allocating compressible memory
|
||||
checkCudaErrors(allocateCompressible((void **)&x, size, true));
|
||||
checkCudaErrors(allocateCompressible((void **)&y, size, true));
|
||||
bool compressibleZbuf = 0;
|
||||
if ((major == 8 && minor == 0) || (major == 8 && minor == 6)) {
|
||||
// On SM 8.0 and 8.6 GPUs compressible buffer can only be initialized
|
||||
// through cudaMemcpy.
|
||||
printf("allocating non-compressible Z buffer\n");
|
||||
checkCudaErrors(allocateCompressible((void **)&z, size, false));
|
||||
compressibleZbuf = 0;
|
||||
}
|
||||
else {
|
||||
checkCudaErrors(allocateCompressible((void **)&z, size, true));
|
||||
compressibleZbuf = 1;
|
||||
}
|
||||
|
||||
printf("Running saxpy on %zu bytes of Compressible memory\n", size);
|
||||
|
||||
const float a = 1.0f;
|
||||
const float init_val = 1.0f;
|
||||
launchSaxpy(a, x, y, z, n, init_val, compressibleZbuf);
|
||||
|
||||
checkCudaErrors(freeCompressible(x, size, true));
|
||||
checkCudaErrors(freeCompressible(y, size, true));
|
||||
checkCudaErrors(freeCompressible(z, size, true));
|
||||
|
||||
printf("Running saxpy on %zu bytes of Non-Compressible memory\n", size);
|
||||
// Allocating non-compressible memory
|
||||
checkCudaErrors(allocateCompressible((void **)&x, size, false));
|
||||
checkCudaErrors(allocateCompressible((void **)&y, size, false));
|
||||
checkCudaErrors(allocateCompressible((void **)&z, size, false));
|
||||
|
||||
launchSaxpy(a, x, y, z, n, init_val, compressibleZbuf);
|
||||
|
||||
checkCudaErrors(freeCompressible(x, size, false));
|
||||
checkCudaErrors(freeCompressible(y, size, false));
|
||||
checkCudaErrors(freeCompressible(z, size, false));
|
||||
|
||||
printf("\nNOTE: The CUDA Samples are not meant for performance measurements. "
|
||||
"Results may vary when GPU Boost is enabled.\n");
|
||||
return EXIT_SUCCESS;
|
||||
}
|
||||
Reference in New Issue
Block a user